@@ -9,35 +9,71 @@ from paste import httpserver, reloader
 from clue_script import Command, __version__
 
 
+prog_prefix = os.path.basename(sys.argv[0])
+
+
 class ReloadableServerCommand(Command):
-    '''Run a reloadable development web server.
+    '''Launch web server to serve this application.
     '''
 
     _logger_name = __package__ + '.server'
 
     __name__ = 'runserver'
 
-    parser = argparse.ArgumentParser()
-    parser.add_argument('-i', '--host',
-                        help=('Host/IP to listen on, specify 0.0.0.0 '
-                              'for all available interfaces'),
-                        default='0.0.0.0', metavar='host')
-    parser.add_argument('-p', '--port', type=int,
-                        help='Port to listen on, defaults to 8080',
-                        default=8080, metavar='port')
-    parser.add_argument('--with-reloader', action='store_true',
-                        help='Watch for code changes and restart as necessary',
-                        default=False)
+    defaults = {
+        'host': '0.0.0.0',
+        'port': '8080',
+        'with_reloader': 1,
+        }
 
-    def __init__(self, app_factory, logger=None):
+    use_rfoo = False
+    rfoo_namespace = None
+
+    def __init__(self, app_factory=None, application=None, logger=None):
         super(ReloadableServerCommand, self).__init__(logger)
-        self.app_factory = app_factory
+        if application is not None:
+            self.application = application
+        elif app_factory is not None:
+            self.application = app_factory()
+        else:
+            raise ValueError('Must specify one of "app_factory" or "app"')
+
+        self.rfoo_namespace = {}
+
+        self.parser = parser = \
+            argparse.ArgumentParser(prog=prog_prefix + ' ' + self.__name__,
+                                    description=self.__doc__)
+        parser.add_argument('-i', '--host',
+                            help=('Host/IP/Interface to listen on, '
+                                  'specify 0.0.0.0 '
+                                  'for all available interfaces '
+                                  '(default: %(default)s)'),
+                            default=self.defaults['host'], metavar='host')
+        parser.add_argument('-p', '--port', type=int,
+                            help='Port to listen on (default: %(default)s)',
+                            default=self.defaults['port'], metavar='port')
+        parser.add_argument('--with-reloader', nargs='?',
+                            help=('1 to watch for code changes and restart '
+                                  'as necessary (default: %(default)s)'),
+                            default=self.defaults['with_reloader'])
 
     def run(self, argv):
         ns = self.parser.parse_args(argv)
-        app = self.app_factory()
-        runner = WSGIAppRunner(application=app, host=ns.host,
-                               port=ns.port, logger=self.logger)
+        ns.with_reloader = bool(int(ns.with_reloader))
+
+        http_logger = get_http_logger(self.logger)
+        if len(http_logger.handlers) == 0:
+            handler = logging.StreamHandler()
+            handler.setFormatter(logging.Formatter('%(message)s'))
+            http_logger.addHandler(handler)
+            http_logger.propagate = False
+
+        runner = WSGIAppRunner(application=self.application,
+                               host=ns.host,
+                               port=ns.port,
+                               logger=self.logger,
+                               use_rfoo=self.use_rfoo,
+                               rfoo_namespace=self.rfoo_namespace)
         runner.wsgi_serve(with_reloader=ns.with_reloader)
 
 
@@ -46,12 +82,17 @@ class WSGIHandler(httpserver.WSGIHandler, object):
         self.logger = logger
         super(WSGIHandler, self).__init__(*args, **kwargs)
 
-    def wsgi_execute(self, environ=None):
-        super(WSGIHandler, self).wsgi_execute(environ=environ)
+    def wsgi_start_response(self, status, response_headers, exc_info=None):
+        res = super(WSGIHandler, self).wsgi_start_response(
+            status, response_headers, exc_info)
         environ = self.wsgi_environ
-        self.logger.info('%s %s%s' % (environ['REQUEST_METHOD'],
-                                      environ['SCRIPT_NAME'],
-                                      environ['PATH_INFO']))
+        pieces = status.split(' ')
+        status = pieces[0]
+        self.logger.info('%s %s%s %s' % (environ['REQUEST_METHOD'],
+                                         environ['SCRIPT_NAME'],
+                                         environ['PATH_INFO'],
+                                         status))
+        return res
 
 
 class WSGIServer(httpserver.WSGIServer, object):
@@ -73,15 +114,28 @@ class WSGIServer(httpserver.WSGIServer, object):
         return WSGIHandler(self.logger, *args, **kwargs)
 
 
+def get_http_logger(logger=None):
+    http_name = getattr(logger, 'name', '')
+    if http_name:
+        http_name += '.http'
+    else:
+        http_name = 'clue_script.http'
+    return logging.getLogger(http_name)
+
+
 class WSGIAppRunner(object):
 
     _reloader_key = 'CLUE_SCRIPT_RELOADER'
+    rfoo_port = 54321
 
-    def __init__(self, application, host, port, logger=None):
+    def __init__(self, application, host, port, logger=None,
+                 use_rfoo=False, rfoo_namespace={}):
         self.application = application
         self.host = host
         self.port = port
         self.logger = logger or logging.getLogger('clue_script.unused')
+        self.use_rfoo = use_rfoo
+        self.rfoo_namespace = rfoo_namespace
 
     def wsgi_serve(self, with_reloader):
         if not with_reloader or self._reloader_key not in os.environ:
@@ -92,8 +146,14 @@ class WSGIAppRunner(object):
             reloader.install()
 
         if self._reloader_key in os.environ or not with_reloader:
+            if self.use_rfoo:
+                from rfoo.utils import rconsole
+                rconsole.spawn_server(self.rfoo_namespace, self.rfoo_port)
+                self.logger.info('Rfoo listening on port %i' % self.rfoo_port)
+            self.logger.info('http ready')
+
             server = WSGIServer(self.application, self.host,
-                                self.port, self.logger)
+                                self.port, get_http_logger(self.logger))
             server.serve_forever()
             return
 
